@@ -10,8 +10,8 @@ setMethod("expand", "CollapsedVCF",
             fxd <- fixed(x)
             fxd$ALT <- unlist(alt(x), use.names=FALSE)
             if (!is.null(geno(x)$AD))
-              geno(x)$AD <- .expandAD(geno(x)$AD, nrow(x), ncol(x))
-            return(VCF(rowData=rowData(x), colData=colData(x), 
+              geno(x)$AD <- .expandAD(geno(x)$AD, nrow(x), ncol(x), elt)
+            return(VCF(rowData=.stripRowData(rowData(x)), colData=colData(x), 
                        exptData=exptData(x), fixed=fxd, 
                        info=.unlistAltInfo(x), geno=geno(x), 
                        ..., collapsed=FALSE))
@@ -26,18 +26,17 @@ setMethod("expand", "CollapsedVCF",
         ## geno 
         gexp <- .expandGeno(x, hdr, elt, idx)
         ## rowData
-        if (is.null(rowData(x)$paramRangeID)) {
-            rdexp <- rowData(x)[idx, ]
-            mcols(rdexp) <- NULL
-        } else {
-            rdexp <- rowData(x)[idx, "paramRangeID"]
-        }
+        rdexp <- .stripRowData(rowData(x)[idx,])
         ## exptData, colData untouched
         VCF(rowData=rdexp, colData=colData(x), exptData=exptData(x),
             fixed=fexp, info=iexp, geno=gexp, ..., collapsed=FALSE)
     }
 )
 
+.stripRowData <- function(x) {
+  x[,intersect(colnames(mcols(x)), "paramRangeID")]
+}
+
 .expandGeno <- function(x, hdr, elt, idx)
 {
     gvar <- geno(x)
@@ -82,7 +81,7 @@ setMethod("expand", "CollapsedVCF",
             }
         } else {
             ## 'Number' is '.'
-            gvar$AD <- .expandAD(AD, length(idx), ncol(x))
+            gvar$AD <- .expandAD(AD, length(idx), ncol(x), elt)
         }
     }
     isA <- names(gvar) %in% rownames(ghdr)[isA]
@@ -98,9 +97,17 @@ setMethod("expand", "CollapsedVCF",
 }
 
 ## returns an array of REF,ALT pairs
-.expandAD <- function(AD, idxlen, xcols)
+.expandAD <- function(AD, idxlen, xcols, elt)
 {
     if (is.list(AD)) {
+        emptyAD <- elementLengths(AD) == 0L
+        if (all(elementLengths(elt) == 1L)) {
+            AD[emptyAD] <- list(c(NA_integer_, NA_integer_))
+        } else {
+            emptypart <- PartitioningByWidth(elt[emptyAD])
+            na <- rep(NA_integer_, sum(width(emptypart)))
+            AD[emptyAD] <- as.list(relist(na, emptypart))
+        }
         adpart <- PartitioningByWidth(AD)
         nalt <- width(adpart) - 1L
         if (sum(nalt) != idxlen*xcols)
@@ -147,11 +154,16 @@ setMethod("expand", "CollapsedVCF",
 }
 
 .unlistAltInfo <- function(x) {
-  hdr <- exptData(x)$header
-  inms <- rownames(info(hdr))[info(hdr)$Number == "A"]
-  ivar <- info(x)
-  ivar[inms] <- lapply(ivar[inms], drop)
-  ivar
+    ivar <- info(x)
+    if (ncol(ivar) == 0L)
+        return(ivar)
+    hdr <- header(x)
+    inms <- rownames(info(hdr))[info(hdr)$Number == "A"]
+    inms <- inms[inms %in% names(ivar)]
+    if (length(inms))
+        return(ivar)
+    ivar[inms] <- lapply(ivar[inms], drop)
+    ivar
 }
 
 setMethod("expand", "ExpandedVCF",
